# -*- coding: utf-8 -*-
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
__doc__="""ATCTMinimumSample Product for Plone (ATCT)
$Id: smallcontent.py 96 2006-03-09 16:44:28Z taka $"""

__version__='$Rev$'[11:-2]
__product_version__ = "Small 0.0.1"
__author__  = 'Shimizukawa'
__docformat__ = 'plaintext'


# 以下のコードで使用するモジュール・関数等をimportします。
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder
from Products.Archetypes.public import Schema,registerType
from Products.ATContentTypes.content.base import ATCTFolder
from Products.ATContentTypes.content.folder import finalizeATCTSchema
from AccessControl import ClassSecurityInfo

#このプロダクトで使用するField
from Products.Archetypes.public import StringField,IntegerField,\
                                   LinesField,ReferenceField,BooleanField
#このプロダクトで使用するWidget
from Products.Archetypes.public import StringWidget,TextAreaWidget,\
                               IntegerWidget,SelectionWidget,BooleanWidget,\
                               ReferenceWidget

# このプロダクトの基本設定値
from Products.ATCTSmallSample.config import *
from Products.Archetypes.public import IntDisplayList

# このコンテンツタイプで使用するデータの種別やUIの表示方法をスキーマとして
# 定義します。定義時に、完全に新しく作るのではなく、継承元クラスのスキーマ
# をコピーし、独自拡張を行います。ATCTとして作る場合は、最低限
# ATContentTypes.content.schemata.ATContentTypeSchema を拡張するようにして
# ください。
# 各FieldやWidgetについては
# http://plone.org/products/archetypes/documentation/manual/quickref
# を参照してください。FieldやWidgetに指定できるかなりの数のパラメータについ
# てドキュメント化されています。
# ただし、最新Documentは常にコードとともにあります。Archetypes/Field.py,
# Widget.py を読むことも参考になるでしょう。
SmallContentSchema = ATCTFolder.schema.copy() + Schema((

    # Plone上の別のオブジェクトへの参照
    ReferenceField(# 変数名として利用されるid
                   'rel_val',
                   relationship='any_rel',
                   # 参照リストに表示する参照先オブジェクトの種類を限定する場合、portal_typeをタプルで指定します
                   allowed_types=(),
                   # 複数選択可能か？
                   multivalued=0,
                   # 必須項目か？
                   required=True,
                   widget=ReferenceWidget(# UIで表示される名前
                                          label='関連づけ',
                                          # UIで表示されるTipHelp
                                          description='Plone内のオブジェクトにリンクします')
                 ),

    # TrueかFalseを保存します
    BooleanField('bool_val',
                 #新規作成時のデフォルト値。スキーマを後から追加した場合に既存のオブジェクトに適用されます
                 default=False,
                 required=True,
                 widget=BooleanWidget(label='真偽値',
                                      description='真偽値を保持します',
                                      ),
                 ),

    IntegerField('stock',
                 default=0,
                 required=True,
                 widget=IntegerWidget(label='Int値(自由入力)',
                                      description='任意の数値を自由に入力します',
                                      ),
                 ),

    IntegerField('int_val',
                 default=0,
                 # 選択型の場合に使用される選択肢を与えます。
                 # DisplayList, IntDisplayListはタプルのリストで初期化され、
                 # 各タプルは、HTMLのフォームで渡されるvalueとUIとして表示
                 # される文字列、で構成されます。[(value,display), ...]
                 vocabulary=IntDisplayList([(i, i) for i in range(0, 101)]),
                 widget=SelectionWidget(label='Int値(選択)',
                                        description='0～100までの数値を選択します',
                                        # 表示形式のドロップダウン・ラジオボタン
                                        # 等を指定します。'flex'を指定すると選択肢
                                        # 数から自動的に決定されます。
                                        format='select',
                                        ),
                 ),

    ))




#Finalize schema definition
# この処理で、ATCT特有の「関連コンテンツ」「このアイテムでの議論を許可する」
# スキーマを追加する。
# ATContentTypes.content.schemata.finalizeATCTSchema を参照。
finalizeATCTSchema(SmallContentSchema)


# ATCTFolderを継承した新しいコンテンツタイプとなるクラスを定義します。
# ATCTFolderを継承してるので、このコンテンツタイプはフォルダのようにコンテ
# ナに他のコンテンツタイプなどのオブジェクトを格納することが出来ます。
class SmallContent(ATCTFolder):
    """
        create SmallContent instance.

    >>> from Products.ATCTSmallSample.content.smallcontent import SmallContent
    >>> small = SmallContent('small')
    >>> small.meta_type
    'SmallContent'

        test IInlineObject interface existing.

    >>> from Products.COREBlog2.interfaces.IInlineObject import IInlineObject
    >>> bool(IInlineObject.isImplementedBy(small))
    True

    """
    # 上記はDocTestに使用されるテストコード 兼 使い方マニュアルです。実際に
    # はコンテンツタイプのクラスから上記のようにインスタンスを作ることは無い
    # 思いますが、クラスからインスタンスを作るような場合、テストコード＝利用
    # 例ヘルプとなります。
    # 上記コメントをDocTestとして実行しているのは、
    # ATCTSmallSample/tests/testDocTest.py です。

    # 以下、クラス変数として必要な値を定義します。

    # スキーマ定義を登録
    schema         = SmallContentSchema

    # ナビや追加メニューなどに表示されるアイコンを指定。アイコンはどこからで
    # もアクセスできるところに置いておきます(skins/ATCTSmallSample/ の下)
    content_icon   = 'smallcontent.gif'

    # Zopeのメタタイプ。Zopeインスタンス上でこの名前が重複するとZopeが起動し
    # ないなどの問題が発生するので、ある程度以上長い名前にしておくと良いでしょ
    # う。また、混乱を避けるためにportal_typeやarchetype_nameにもmeta_typeと
    # 同じ名前を設定しておくことを勧めます。
    meta_type      = 'SmallContent'

    # Plone上のメタタイプ。Plone上でコンテントのインスタンスを追加する場合
    # にinvokeFactory()に指定する名前。Ploneが2.1になったときにArchetypes
    # ベースの実装に移行し、DocumentなどがATDocumentというmeta_typeになった
    # が、portal_type名は変わっていない。meta_typeを変える事でZope上にある
    # 既存のmeta_type=Documentなオブジェクトへの影響を無くし、portal_type
    # を変えないことで、既存のPlone向けプロダクトへの影響を無くしている。
    portal_type    = 'SmallContent'

    # Archetype名。PloneのUI上でインスタンス追加メニューに表示される名前で、
    # i18nの翻訳対象となる文字列です。meta_typeやportal_typeを後から書き換
    # えると既存のインスタンスなどへの影響が大きいため、表示名として使う為
    # に用意されています。
    archetype_name = 'SmallContent'

    #
    immediate_view = 'base_view'

    # suppl_viewsでViewの複数選択を可能とする場合の初期Viewを指定します。
    default_view   = 'base_view'

    # コンテンツを表示するためのViewを複数選択可能とする場合に、選択肢とな
    # るメソッド名やPageTemplate名をタプルで列挙します(PageTemplateの場合も
    # オブジェクト名を指定する必要があるので拡張子は不要です)。
    suppl_views    = ()

    #
    typeDescription= 'A small content.'

    #
    typeDescMsgId  = 'small_content_description'

    #
    assocMimetypes = ()

    #
    assocFileExt   = ()

    #
    cmf_edit_kws   = ()

    # Ploneサイトの任意の場所にインスタンスを作成しても良いか？Falseを指定
    # すると任意の場所では追加メニューに表示されなくなる。その代わり、他の
    # コンテンツタイプでallowed_content_typesにこのコンテンツのportal_type
    # を指定すれば、そのコンテンツタイプ直下には追加可能となる。または、プ
    # ログラムコード上からのみ追加したいコンテンツタイプとする事も出来る。
    global_allow   = True

    # コンテンツがフォルダリッシュ(コンテナ型)クラスの場合に、サブコンテン
    # ツを追加することが出来る。Falseを設定すると追加可能なコンテンツタイプ
    # を制限しない。Trueを設定した場合はallowed_content_typesに指定したコン
    # テンツタイプのみをサブコンテンツに登録できるようになる。
    filter_content_types = False

    # filter_content_typesをTrueに設定した場合に有効。詳しくは
    # filter_content_typesの説明を参照してください。
    allowed_content_types = ()

    # Plone-2.1の標準設定では、インスタンス新規作成時にidを指定するFormが表
    # 示されません。このためidは自動生成されますが、
    # __at_rename_after_creation にTrueを設定した場合、インスタンスの新規作
    # 成時に、titleプロパティーの値からidを自動生成します。Falseの場合は
    # titleプロパティーを使わずに自動生成します。
    # idを指定する設定の場合にidが指定された場合は、指定されたidを使用する
    # ため、自動生成は行われません。
    _at_rename_after_creation = True


    # 実装しているインターフェースを定義します。ここでは継承元が実装してい
    # るインターフェースに、マーカーインターフェースに分類される、フォルダ
    # 型クラスをPloneUI上でフォルダとして扱わない(コンテンツタブが表示され
    # ない、等)ようにする INonStructuralFolder を指定しています。
    __implements__ = (ATCTFolder.__implements__, INonStructuralFolder, )

    # implement for coreblog IInlineObject interface
    try:
        # COREBlog2が定義しているIInlineObjectインターフェースを
        # __imprements__に追加します。COREBlog2がProductsに無い場合は冷害が
        # 発生しますが、コードは問題なく実行されます。
        # IInlineObjectインターフェースのメソッドを実装すると、
        # IInlineObject実装オブジェクトを扱えるコンテンツ(COREBlog2等)にイ
        # ンライン表示することが出来るようになります。
        from Products.COREBlog2.interfaces.IInlineObject import IInlineObject
        __implements__ += (IInlineObject,)
    except:
        pass

    # クラスの各メソッドのアクセス権を設定するためのユーティリティーですが、
    # SmallContentでは非Publicにするべきメソッドが無かったため、利用してい
    # ません。
    security = ClassSecurityInfo()

    # Plone-2.1では、フォルダのデフォルトビューを指定することが出来ます。つ
    # まり、index_html等が無い場合にフォルダ内のオブジェクト一覧を表示する
    # というデフォルト動作の代わりに、デフォルトビューになれるオブジェクト
    # のViewを表示することが出来ます。(CMFContentPanelsなど)
    # この関数の返値をTrueとすることで、デフォルトビューに設定できるように
    # なります。
    def canSetDefaultPage(self):
        """Can this content set default page for folder?"""
        return False

    # IInlineObjectのメソッドです。Trueを返した場合、オブジェクトは呼出元コ
    # ンテンツにインライン表示されます。インライン表示するために必要な
    # PageTemplate名などをgetInlineViewメソッドで提供する必要があります。
    # Falseを返した場合は添付表示となり、コンテンツの末尾に表示されます。
    # Trueの場合と同様に、getAttachementViewメソッドで表示方法を記述した
    # PageTemplate名などを提供する必要があります。
    def isInlineObject(self):
        """Is this content inline object or not?

        >>> small.isInlineObject()
        True

        """
        return True

    # IInlineObjectのメソッドです。isInlineObject()でTrueを返した場合に呼び
    # 出されます。インライン表示するために必要な、表示用PageTemplateと、
    # metalのマクロ名をタプルで返します。
    def getInlineView(self):
        """For IInlineObject's inline view, if isInlineObject return True.

        >>> small.getInlineView()
        ('smallcontent_inline_view', 'inline_view')

        """

        templ = 'smallcontent_inline_view'
        macro = 'inline_view'
        return templ, macro

    # IInlineObjectのメソッドです。isInlineObject()でFalseを返した場合に呼
    # び出されます。添付表示するために必要な、表示用PageTemplateと、metalの
    # マクロ名をタプルで返します。
    def getAttachmentView(self):
        """For IInlineObject's attachement view, if isInlineObject return False.

        >>> small.getAttachmentView() is None
        True

        """
        pass


# このモジュールがimportされた時に実行される、Archetypesへのコンテンツタイ
# プ登録関数です。ここで登録されたクラスとプロジェクト名は
# ATCTSmallSample/__init__.pyでprocess_types()で取得したり、や
# Extentions/install.pyでlistTypes()で取得しています。
registerType(SmallContent, PROJECTNAME)

