# -*- coding: utf-8 -*-
#$Id: ATCTSSTestCase.py 96 2006-03-09 16:44:28Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#
# ATCTSSTestCase test setup
#
# ATCTSSTestCase.py は全テストで使用される共通の実装を記述しています(SSは
# SmallSampleの略です)。

from Products.PloneTestCase import PloneTestCase


# 各テスト実装モジュールは、このモジュールをimportするため、以下のインスト
# ール処理を再度実装する必要はありません。

# Zope/Ploneのテストを行うためには、テストの実行に必要な(いま実装しているプ
# ロダクトが依存している)プロダクトを仮想Zopeインスタンスにインストールする
# 必要があります。Zope本体に同梱されている基本的なプロダクトとPlone本体に同
# 梱されている基本的なプロダクトについては、自動的にZopeにインストールされ
# るため、ここではATCTSmallSampleだけを明示的にZopeにインストールします。
PloneTestCase.installProduct('ATCTSmallSample')

# Zopeへのインストールと同様に、仮想Plone環境にも必要なプロダクトを"プロダ
# クトを追加・削除"で行うように追加する必要があります。ここでもPloneの動作
# に必要なプロダクトはPloneTestCase/setup.pyで動的にインストールされるため、
# ATCTSmallSampleを明示的にインストールします。
PloneTestCase.setupPloneSite(products=['ATCTSmallSample',])




# 全てのテスト実装クラスは、PloneTestCaseクラスを継承します。テスト実装クラ
# スでafterSetUp(), beforeTearDown()などの関数を実装することにより、各テス
# ト毎に必要な初期化と後始末を行うことが出来ますが、他にもさまざまなテスト
# で利用したいユーティリティー的な関数が必要となる場合が多々あります。そこ
# で、PloneTestCaseを継承した中間基底クラスを用意してここにユーティリティー
# 関数を用意します。各テスト実装クラスはこの中間クラスを継承することにしま
# す。
class ATCTSSTestCase(PloneTestCase.PloneTestCase):

    # 各テスト実装クラスでよく使われる名前などをクラス変数として用意します。
    defaultTitle = 'Default Testing Title'

    # UnitTestにおいて、一般的には初期化関数がsetUp, 後始末関数がtearDownで
    # すが、これらはZopeTestCaseが環境の用意とロールバックに使用しています。
    # そのため、ZopeTestCaseやPloneTestCaseを使用したテストコードを作成する
    # 場合は、これらの代わりにafterSetUp, beforeTearDownを使用します。呼び
    # 出されるタイミングについて詳しくはZopeTestCase-0.9.8(or Zope-2.9.0以
    # 降のZopeTestCase)に同梱されている
    # lib/python/Testing/ZopeTestCase/doc/TIMELINES.txt を参照してください。
    def afterSetUp(self):
        # Plone環境が作成されたらSmallContentインスタンスをデフォルト名で作
        # 成します。作成位置として指定しているself.folderは個人フォルダを指
        # しています。
        self._createSmallContent(self.folder)

    # 共通で使用されるSmallContentインスタンスを作成するユーティリティー関
    # 数です。
    def _createSmallContent(self, folder, id='small', title=defaultTitle):
        """Creates and returns a refence to a SmallContent.
        This method publishes a SmallContent instance under folder.  It fills in
        all of the standard properties."""

        # インスタンスを作成するためには、作成対象フォルダのinvokeFactoryメ
        # ソッドを呼び出します。
        folder.invokeFactory(# 第一引数はportal_typesに登録されている
                             # portal_typeを指定。meta_typeではない
                             'SmallContent',
                             # オブジェクトのidを指定
                             id=id,
                             # オブジェクトのタイトルを指定
                             title=title,
                             # オブジェクトの概要を指定
                             description='A SmallContent instance for unit tests.',
                            )

        # 作成したSmallContentインスタンスをフォルダから取得します。
        small = getattr(folder, id)
        # 作成したインスタンスにworkflowのinitial stateの設定を適用します。
        self.portal.portal_workflow.doActionFor(small, 'submit')
        # 作成したインスタンスオブジェクトを返します。
        return small

    # プロダクトのアンインストールを行うユーティリティー関数です。プロダク
    # トのアンインストールテストなどのために利用します。ATCTSmallSampleでは、
    # testSetup.TestUninstallSmallContentで使用しています。
    def uninstallProduct(self, product_name):
        # Removes a product
        tool = getattr(self.portal, 'portal_quickinstaller')
        if tool.isProductInstalled(product_name):
            tool.uninstallProducts([product_name])

