# -*- coding: utf-8 -*-
#$Id: testDocTest.py 96 2006-03-09 16:44:28Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#
# ATCTSSTestCase DocTests
#

# このテストモジュールでは、任意のモジュール内に記述されたDocTestを実施する
# テストスイートを作成します。
#
# DocTestは、PythonのDocString（変数に代入されていない文字列）のうち、
# Pythonインタラクティブモード（対話コンソール）のように'>>>'で始まる行の部
# 分を ``テスト実行コード`` として解釈・実行するテストです。特にユーティリ
# ティー的な、環境に依存しない関数のDocStringに記述することで、関数の使い方
# のヘルプも兼ねることが出来、ドキュメント・テスト・実装を一カ所に集約する
# ことが出来ます。
#
# 詳しくはZopeTestCase-0.9.8(or Zope-2.9.0以降のZopeTestCase)に同梱されてい
# る lib/python/Testing/ZopeTestCase/zopedoctest/ZopeDocTest.txt を参照して
# ください（このドキュメント自体もDocTestになっていて、testZopeDocTest.pyか
# ら実行されています）。
#
# 一つ注意しなければ行けないのは、DocTestの実行順序です。ZopeDocFileSuiteを
# 使った場合、以下の順にDocStringが実行されます。
#
#   1. ZopeDocFileSuite('example.py', setUp=setUp) で設定したsetUp関数の
#      DocString
#   2. example.py内の全てのDocString
#
# 各モジュール内のmethodのDocString毎に上記で設定したsetUp/tearDownが呼ばれ
# るわけではなく、ファイル内の全てのDocStringがクラスや関数などに関係なく上
# から順番に実行されます。


# testSetup.pyと同一コードについては説明を省略します。詳しくはtestSetup.py
# を参照してください。

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))


# このsetUpは、各モジュールのDocString実行前に呼び出されます。ここでは2つの
# 事前処理を行っています。
#
# 1. コンテンツタイプクラスのインスタンスを作成するために必要な
#    importAcquisiion,などの環境の用意
#
# 2. COREBlog2 import時に標準出力に出力される文字列をDocTestが受け取ってし
#    まうのを防ぐための事前import処理。
def setUp(self):
    '''This method will run after the test_class' setUp.

    >>> import Acquisition
    >>> from Globals import INSTANCE_HOME
    '''
    try:
        # disable COREBlog2 standard output.
        import Products.COREBlog2
    except:
        pass


def test_suite():
    from Testing.ZopeTestCase import ZopeDocTestSuite
    from Testing.ZopeTestCase import ZopeDocFileSuite
    from unittest import TestSuite
    return TestSuite((
        # 自己モジュール内のDocTestをTestSuiteとして登録します
        ZopeDocTestSuite(setUp=setUp),

        # 指定ファイルのDocTestをTestSuiteとして登録します
        ZopeDocFileSuite('../content/smallcontent.py', setUp=setUp),
    ))


if __name__ == '__main__':
    framework()

