# -*- coding: utf-8 -*-
#$Id: testSetup.py 96 2006-03-09 16:44:28Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#
# ATCTSmallSample Setup tests
#

# ATCTSmallSampleで最初に作成したのは、PloneQuickInstallerの処理をテストす
# るtestSetup.pyです。モジュール内には ``インストール処理`` と ``アンインス
# トール処理`` が正しく行われたかを確認するテストクラスがそれぞれあります。
# テストクラスはメソッドとして ``test`` で始まる名前の関数を持っています。
# これらの各関数が ``TestCase`` で、TestCaseの集まりが ``TestSuite`` とよば
# れます。
#
# 各TestCaseとしては、Extentions/Install.py で行っている処理に対するテスト
# が記述されています。具体的には、
#
# - portal_typesにコンテンツタイプが登録されたか
# - DynamicViewFTIに変換されたか
# - portal_factoryがコンテンツタイプに対して有効になっているか
# - skins以下のスキンがskinレイヤーとして登録されたか
# - portal_cssにcssオブジェクトが登録されたか
#
# を確認しています。そして、アンインストール後には上記が全て無効になっている
# かを確認します。

# 以下の3行は、このモジュールが単体実行された場合にframework.pyをこのモジュ
# ール内のコードとして呼び出すためにあります。runalltest.pyから実行される場
# 合には以下3行は動作せず、このモジュールのtest_suite()が呼び出されます。
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from AccessControl import Unauthorized
from Products.ATCTSmallSample.tests.ATCTSSTestCase import ATCTSSTestCase
from Products.ATCTSmallSample.config import *

# インストールが正しく完了しているかを確認するテストスイートです。インスト
# ール処理などはATCTSSTestCaseで行っているので、ここではATCTSSTestCaseの
# importと継承のみで準備が完了します。
class TestInstallSmallContent(ATCTSSTestCase):

    # portal_type が登録されているかを確認します。テストメソッドに
    # DocStringを書くと、テスト失敗時に、失敗したテスト関数名の代わりに
    # DocStringが出力されます。
    def testSmallContentPortalType(self):
        """SmallContent in portal_types"""
        self.failUnless('SmallContent' in self.portal.portal_types.objectIds())

    # SmallContentがDynamicViewFTIに変換できているかを確認します。確認方法
    # として、interfaceのimplement状態を調べています。
    def testSmallContentDynamicViewFTI(self):
        """SmallContent implemented by IDynamicViewTypeInformation"""
        small = getattr(self.folder, 'small')
        from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
        info = self.portal.portal_types['SmallContent']
        self.failUnless(IDynamicViewTypeInformation.isImplementedBy(info))

    # portal_factoryが有効になっているかを確認します。portal_factoryの有効
    # 化はプロダクト全体ではなくコンテンツタイプ毎の実装となるため、有効化
    # したコンテンツタイプそれぞれについてテストが必要です。
    def testSmallContentPortalFactoryEnabled(self):
        """PortalFactory enabled on SmallContent"""
        from Products.CMFCore.utils import getToolByName
        factory = getToolByName(self.portal, 'portal_factory')
        types = factory.getFactoryTypes().keys()
        self.failUnless('SmallContent' in types)

    # portal_skinsへのskinレイヤー登録を確認します。ATCTSmallSampleは全ての
    # SkinSelectionに対してレイヤーを登録しています。(レイヤーの登録と
    # FolderView登録は異なることに注意してください。)
    # ATCTSmallSampleはskinsディレクトリ以下にプロダクト名と同名のskinレイ
    # ヤーが一つだけあります。複数のスキンレイヤーをインストールするプロダ
    # クトの場合は、それぞれのレイヤー名について同様のテストを行う必要があ
    # ります。
    def testSmallContentSkin(self):
        """SmallContent skin path in portal_skins"""
        for selection in self.portal.portal_skins.getSkinSelections():
            path = self.portal.portal_skins.getSkinPath(selection)
            self.failUnless(PROJECTNAME in path)

    # portal_cssへのcssオブジェクトの登録を確認します。ここでは
    # smallcontent.cssの登録のみを確認していますが、enable状態などの確認を
    # 行っても良いでしょう。
    def testSmallContentPortalCSS(self):
        """smallcontent.css in portal_css"""
        cssreg = self.portal.portal_css
        stylesheet_ids = cssreg.getResourceIds()
        self.failUnless('smallcontent.css' in stylesheet_ids)


# アンインストールが正しく完了しているかを確認するテストスイートです。ここ
# ではskinレイヤーが正しく削除されているかを主目的にテストしていますが、
# Tool関連をploneルートにインストールする場合などにはそれらの削除の確認など
# をテストします。
class TestUninstallSmallContent(ATCTSSTestCase):

    # ATCTSmallSampleスキンレイヤーの削除を確認します。基本的に存在しない
    # skinレイヤーがあっても問題にはなりませんが、portal_skinsのpropertyで
    # skinレイヤーの編集保存を行った場合に存在しないレイヤーを警告され、そ
    # れらを取り除くまで保存できなくなってしまいます。後始末はしっかりとや
    # りましょう。
    # 複数skinを登録するプロダクトの場合はテストの修正が必要です。
    def testSmallContentSkin(self):
        """SmallContent skin path not in portal_skins"""
        self.uninstallProduct(PROJECTNAME)
        for selection in self.portal.portal_skins.getSkinSelections():
            path = self.portal.portal_skins.getSkinPath(selection)
            self.failUnless(PROJECTNAME not in path)

    # Plone-2.1.2では、portal_cssに登録したcssはプロダクトの削除とともに登
    # 録が解除されるようです。
    def testSmallContentPortalCSS(self):
        """smallcontent.css not in portal_css"""
        self.uninstallProduct(PROJECTNAME)
        cssreg = self.portal.portal_css
        stylesheet_ids = cssreg.getResourceIds()
        self.failUnless('smallcontent.css' not in stylesheet_ids)


# runalltests.pyから呼び出されるテストスイートの登録関数です。
def test_suite():
    from unittest import TestSuite, makeSuite

    # テストスイートを作成します。ATCTSSTestCaseを継承しているクラスもテス
    # トスイートですが、ここではそれらを束ねる入れ物としてのテストスイート
    # を用意します。
    suite = TestSuite()

    # テストクラスをsuite.addTestで受け取れる型に変換してから追加します。
    suite.addTest(makeSuite(TestInstallSmallContent))
    suite.addTest(makeSuite(TestUninstallSmallContent))

    # 呼出元にテストスイートを返します。
    return suite

if __name__ == '__main__':
    # モジュール内のZopeTestCase継承クラスをテストスイートとして実行します。
    # 対象クラスは自動的に認識するため、明示的な登録は必要ありません。
    framework()
